"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var eventemitter2_1 = require("eventemitter2");
var firebase_key_1 = require("firebase-key");
var json = require("../json");
var lodash = require("../lodash");
var mock_error_1 = require("../mock-error");
var observer_1 = require("../observer");
var mock_document_ref_1 = require("./mock-document-ref");
var mock_firestore_paths_1 = require("./mock-firestore-paths");
var mock_query_snapshot_1 = require("./mock-query-snapshot");
var MockCollectionRef = (function () {
    function MockCollectionRef(options) {
        this.app_ = options.app;
        this.emitters_ = options.emitters;
        this.fieldPath_ = options.fieldPath;
        this.fieldValue_ = options.fieldValue;
        this.firestore_ = options.firestore;
        this.query_ = options.query || { where: [] };
        this.store_ = options.store;
        this.path_ = mock_firestore_paths_1.toPath(options.path);
        mock_firestore_paths_1.validatePath(this.path_);
        var index = this.path_.lastIndexOf("/");
        if (index === -1) {
            this.id_ = this.path_;
            this.parentPath_ = "";
            this.jsonPath_ = mock_firestore_paths_1.toJsonPath(this.path_);
        }
        else {
            this.id_ = this.path_.substring(index + 1);
            this.parentPath_ = this.path_.substring(0, index);
            this.jsonPath_ = json.join(mock_firestore_paths_1.toJsonPath(this.parentPath_), "collections", this.id_);
        }
        this.refEmitter_ = new eventemitter2_1.EventEmitter2();
        this.rootEmitter_ = this.emitters_.root;
        this.sharedEmitter_ = this.emitters_.shared[this.jsonPath_];
        if (!this.sharedEmitter_) {
            this.sharedEmitter_ = new eventemitter2_1.EventEmitter2({ wildcard: true });
            this.emitters_.shared[this.jsonPath_] = this.sharedEmitter_;
        }
        this.sharedEmitter_.onAny(this.sharedListener_.bind(this));
    }
    Object.defineProperty(MockCollectionRef.prototype, "firestore", {
        get: function () {
            return this.firestore_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockCollectionRef.prototype, "id", {
        get: function () {
            return this.id_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockCollectionRef.prototype, "parent", {
        get: function () {
            return this.parentPath_ ? new mock_document_ref_1.MockDocumentRef({
                app: this.app_,
                emitters: this.emitters_,
                fieldPath: this.fieldPath_,
                fieldValue: this.fieldValue_,
                firestore: this.firestore_,
                path: this.parentPath_,
                store: this.store_
            }) : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockCollectionRef.prototype, "path", {
        get: function () {
            return this.path_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockCollectionRef.prototype, "queried_", {
        get: function () {
            return !lodash.isEqual(this.query_, { where: [] });
        },
        enumerable: true,
        configurable: true
    });
    MockCollectionRef.prototype.add = function (data) {
        if (this.queried_) {
            throw mock_error_1.unsupported_("Queries do not support 'add'.");
        }
        var docRef = this.doc(firebase_key_1.key());
        return docRef.set(data).then(function () { return docRef; });
    };
    MockCollectionRef.prototype.doc = function (documentPath) {
        return new mock_document_ref_1.MockDocumentRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: json.join(this.path_, documentPath || firebase_key_1.key()),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.endAt = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { endAt: args }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.endBefore = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { endBefore: args }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.get = function () {
        return Promise.resolve(new mock_query_snapshot_1.MockQuerySnapshot({
            content: this.store_.content,
            ref: this
        }));
    };
    MockCollectionRef.prototype.isEqual = function (other) {
        var otherMock = other;
        return (this.jsonPath_ === otherMock.jsonPath_) && lodash.isEqual(this.query_, otherMock.query_);
    };
    MockCollectionRef.prototype.limit = function (limit) {
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { limit: limit }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.onSnapshot = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var options;
        var observer = observer_1.toObserver.apply(void 0, args);
        if (observer) {
            options = {};
        }
        else {
            var rest = void 0;
            options = args[0], rest = args.slice(1);
            observer = observer_1.toObserver.apply(void 0, rest);
        }
        if (options.includeDocumentMetadataChanges) {
            throw mock_error_1.unsupported_();
        }
        if (options.includeQueryMetadataChanges) {
            throw mock_error_1.unsupported_();
        }
        Promise.resolve().then(function () { return observer.next(new mock_query_snapshot_1.MockQuerySnapshot({
            content: _this.store_.content,
            ref: _this
        })); });
        var listener = function (snapshot) {
            observer.next(snapshot);
        };
        this.refEmitter_.on("snapshot", listener);
        return function () { return _this.refEmitter_.off("snapshot", listener); };
    };
    MockCollectionRef.prototype.orderBy = function (fieldPath, directionStr) {
        if (typeof fieldPath !== "string") {
            throw mock_error_1.unsupported_();
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { orderByDirection: directionStr, orderByField: fieldPath }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.startAfter = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { startAfter: args }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.startAt = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { startAt: args }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.where = function (fieldPath, opStr, value) {
        if (typeof fieldPath !== "string") {
            throw mock_error_1.unsupported_();
        }
        return new MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: this.path_,
            query: tslib_1.__assign({}, this.query_, { where: this.query_.where.concat([{
                        field: fieldPath,
                        operator: opStr,
                        value: value
                    }]) }),
            store: this.store_
        });
    };
    MockCollectionRef.prototype.sharedListener_ = function (eventType, _a) {
        var content = _a.content, previousContent = _a.previousContent;
        this.refEmitter_.emit("snapshot", new mock_query_snapshot_1.MockQuerySnapshot({
            content: content,
            ref: this
        }));
    };
    return MockCollectionRef;
}());
exports.MockCollectionRef = MockCollectionRef;
