"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var eventemitter2_1 = require("eventemitter2");
var json = require("../json");
var lodash = require("../lodash");
var mock_error_1 = require("../mock-error");
var observer_1 = require("../observer");
var mock_collection_ref_1 = require("./mock-collection-ref");
var mock_document_snapshot_1 = require("./mock-document-snapshot");
var mock_firestore_paths_1 = require("./mock-firestore-paths");
var MockDocumentRef = (function () {
    function MockDocumentRef(options) {
        this.app_ = options.app;
        this.emitters_ = options.emitters;
        this.fieldPath_ = options.fieldPath;
        this.fieldValue_ = options.fieldValue;
        this.firestore_ = options.firestore;
        this.store_ = options.store;
        this.path_ = mock_firestore_paths_1.toPath(options.path);
        mock_firestore_paths_1.validatePath(this.path_);
        this.jsonPath_ = mock_firestore_paths_1.toJsonPath(this.path_);
        var index = this.path_.lastIndexOf("/");
        this.id_ = this.path_.substring(index + 1);
        this.parentPath_ = this.path_.substring(0, index);
        this.refEmitter_ = new eventemitter2_1.EventEmitter2();
        this.rootEmitter_ = this.emitters_.root;
        this.sharedEmitter_ = this.emitters_.shared[this.jsonPath_];
        if (!this.sharedEmitter_) {
            this.sharedEmitter_ = new eventemitter2_1.EventEmitter2({ wildcard: true });
            this.emitters_.shared[this.jsonPath_] = this.sharedEmitter_;
        }
        this.sharedEmitter_.onAny(this.sharedListener_.bind(this));
    }
    Object.defineProperty(MockDocumentRef.prototype, "firestore", {
        get: function () {
            return this.firestore_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentRef.prototype, "id", {
        get: function () {
            return this.id_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentRef.prototype, "parent", {
        get: function () {
            return new mock_collection_ref_1.MockCollectionRef({
                app: this.app_,
                emitters: this.emitters_,
                fieldPath: this.fieldPath_,
                fieldValue: this.fieldValue_,
                firestore: this.firestore_,
                path: this.parentPath_,
                store: this.store_
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentRef.prototype, "path", {
        get: function () {
            return this.path_;
        },
        enumerable: true,
        configurable: true
    });
    MockDocumentRef.prototype.collection = function (collectionPath) {
        return new mock_collection_ref_1.MockCollectionRef({
            app: this.app_,
            emitters: this.emitters_,
            fieldPath: this.fieldPath_,
            fieldValue: this.fieldValue_,
            firestore: this.firestore_,
            path: json.join(this.path_, collectionPath),
            store: this.store_
        });
    };
    MockDocumentRef.prototype.delete = function () {
        var _this = this;
        var previousContent = this.store_.content;
        var error = json.findError(this.jsonPath_, this.store_.content);
        if (error) {
            return Promise.reject(error);
        }
        return Promise.resolve().then(function () {
            _this.store_.content = json.remove(_this.store_.content, json.join(_this.jsonPath_, "data"));
            _this.rootEmitter_.emit("content", {
                content: _this.store_.content,
                previousContent: previousContent
            });
        });
    };
    MockDocumentRef.prototype.get = function () {
        return Promise.resolve(new mock_document_snapshot_1.MockDocumentSnapshot({
            content: this.store_.content,
            ref: this
        }));
    };
    MockDocumentRef.prototype.isEqual = function (other) {
        return this.path === other.path;
    };
    MockDocumentRef.prototype.onSnapshot = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var options;
        var observer = observer_1.toObserver.apply(void 0, args);
        if (observer) {
            options = {};
        }
        else {
            var rest = void 0;
            options = args[0], rest = args.slice(1);
            observer = observer_1.toObserver.apply(void 0, rest);
        }
        if (options.includeMetadataChanges) {
            throw mock_error_1.unsupported_();
        }
        Promise.resolve().then(function () { return observer.next(new mock_document_snapshot_1.MockDocumentSnapshot({
            content: _this.store_.content,
            ref: _this
        })); });
        var listener = function (snapshot) {
            observer.next(snapshot);
        };
        this.refEmitter_.on("snapshot", listener);
        return function () { return _this.refEmitter_.off("snapshot", listener); };
    };
    MockDocumentRef.prototype.set = function (data, options) {
        var _this = this;
        var previousContent = this.store_.content;
        mock_firestore_paths_1.validateFields(data);
        data = json.clone(data);
        var error = json.findError(this.jsonPath_, this.store_.content);
        if (error) {
            return Promise.reject(error);
        }
        return Promise.resolve().then(function () {
            if (options && options.merge) {
                _this.store_.content = mergeData(_this.store_.content, json.join(_this.jsonPath_, "data"), _this.fieldValue_, data);
            }
            else {
                _this.store_.content = json.set(_this.store_.content, json.join(_this.jsonPath_, "data"), data);
            }
            _this.rootEmitter_.emit("content", {
                content: _this.store_.content,
                previousContent: previousContent
            });
        });
    };
    MockDocumentRef.prototype.update = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var previousContent = this.store_.content;
        var data;
        if (args.length === 1) {
            data = args[0];
        }
        else {
            data = {};
            for (var a = 0; a < args.length; a += 2) {
                if (typeof args[a] !== "string") {
                    throw mock_error_1.unsupported_();
                }
                data[args[a]] = args[a + 1];
            }
        }
        mock_firestore_paths_1.validateFields(data);
        data = json.clone(data);
        var error = json.findError(this.jsonPath_, this.store_.content);
        if (error) {
            return Promise.reject(error);
        }
        return Promise.resolve().then(function () {
            _this.store_.content = mergeData(_this.store_.content, json.join(_this.jsonPath_, "data"), _this.fieldValue_, data);
            _this.rootEmitter_.emit("content", {
                content: _this.store_.content,
                previousContent: previousContent
            });
        });
    };
    MockDocumentRef.prototype.sharedListener_ = function (eventType, _a) {
        var content = _a.content;
        this.refEmitter_.emit("snapshot", new mock_document_snapshot_1.MockDocumentSnapshot({
            content: content,
            ref: this
        }));
    };
    return MockDocumentRef;
}());
exports.MockDocumentRef = MockDocumentRef;
function mergeData(content, path, fieldValue, data) {
    var deleteFieldValue = fieldValue["delete"];
    lodash.each(data, function (value, key) {
        var slashPath = mock_firestore_paths_1.toSlashPath(key);
        var jsonPath = mock_firestore_paths_1.toJsonPath(json.join(path, slashPath));
        var error = json.findError(jsonPath, content);
        if (error) {
            throw error;
        }
        if (value === deleteFieldValue) {
            content = json.remove(content, jsonPath);
            json.prune(content, jsonPath);
        }
        else {
            content = json.set(content, jsonPath, json.clone(value));
        }
    });
    return content;
}
