"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var json = require("../json");
var mock_error_1 = require("../mock-error");
var mock_firestore_paths_1 = require("./mock-firestore-paths");
var MockDocumentSnapshot = (function () {
    function MockDocumentSnapshot(options) {
        this.content_ = options.content;
        this.ref_ = options.ref;
    }
    Object.defineProperty(MockDocumentSnapshot.prototype, "exists", {
        get: function () {
            return Boolean(this.data());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentSnapshot.prototype, "id", {
        get: function () {
            return this.ref_.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentSnapshot.prototype, "metadata", {
        get: function () {
            return {
                fromCache: false,
                hasPendingWrites: false
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockDocumentSnapshot.prototype, "ref", {
        get: function () {
            return this.ref_;
        },
        enumerable: true,
        configurable: true
    });
    MockDocumentSnapshot.prototype.data = function () {
        var dataPath = json.join(this.ref_.jsonPath_, "data");
        var data;
        if (json.has(this.content_, dataPath)) {
            data = json.get(this.content_, dataPath);
            if (data) {
                data = json.clone(data);
            }
        }
        return data;
    };
    MockDocumentSnapshot.prototype.get = function (fieldPath) {
        if (typeof fieldPath !== "string") {
            throw mock_error_1.unsupported_();
        }
        var slashPath = mock_firestore_paths_1.toSlashPath(fieldPath);
        var dataPath = mock_firestore_paths_1.toJsonPath(json.join(this.ref_.jsonPath_, "data", slashPath));
        var data;
        if (json.has(this.content_, dataPath)) {
            data = json.get(this.content_, dataPath);
            if (data) {
                data = json.clone(data);
            }
        }
        return data;
    };
    return MockDocumentSnapshot;
}());
exports.MockDocumentSnapshot = MockDocumentSnapshot;
