"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var json = require("../json");
var lodash = require("../lodash");
var mock_document_snapshot_1 = require("./mock-document-snapshot");
var MockQuerySnapshot = (function () {
    function MockQuerySnapshot(options) {
        this.content_ = options.content;
        this.ref_ = options.ref;
    }
    Object.defineProperty(MockQuerySnapshot.prototype, "docChanges", {
        get: function () {
            return [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "docs", {
        get: function () {
            var _this = this;
            return lodash.map(this.pairs_(), function (pair) { return new mock_document_snapshot_1.MockDocumentSnapshot({
                content: _this.content_,
                ref: _this.ref_.doc(pair.id)
            }); });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "empty", {
        get: function () {
            return this.size === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "metadata", {
        get: function () {
            return {
                fromCache: false,
                hasPendingWrites: false
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "query", {
        get: function () {
            return this.ref_;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "size", {
        get: function () {
            return this.pairs_().length;
        },
        enumerable: true,
        configurable: true
    });
    MockQuerySnapshot.prototype.forEach = function (callback, thisArg) {
        this.docs.forEach(callback, thisArg);
    };
    MockQuerySnapshot.prototype.pairs_ = function () {
        if (!json.has(this.content_, this.ref_.jsonPath_)) {
            return [];
        }
        var collection = json.get(this.content_, this.ref_.jsonPath_);
        var query = this.ref_.query_;
        var pairs = lodash.map(collection, toPair);
        pairs = lodash.filter(pairs, function (pair) { return Boolean(pair.doc.data); });
        if (query.where.length) {
            pairs = lodash.filter(pairs, wherePredicate(query));
        }
        if (query.orderByField) {
            pairs.sort(orderByFieldComparer(query));
        }
        else {
            pairs.sort(orderByIdComparer(query));
        }
        if ((query.startAfter !== undefined) || (query.startAt !== undefined)) {
            pairs = lodash.filter(pairs, startPredicate(query));
            if (query.limit && (pairs.length > query.limit)) {
                pairs = pairs.slice(0, query.limit);
            }
        }
        else if ((query.endAt !== undefined) || (query.endBefore !== undefined)) {
            pairs = lodash.filter(pairs, endPredicate(query));
            if (query.limit && (pairs.length > query.limit)) {
                pairs = pairs.slice(pairs.length - query.limit, pairs.length);
            }
        }
        else if (query.limit && (pairs.length > query.limit)) {
            pairs = pairs.slice(0, query.limit);
        }
        return pairs;
    };
    return MockQuerySnapshot;
}());
exports.MockQuerySnapshot = MockQuerySnapshot;
function endPredicate(query) {
    return function (pair) {
        var value = getOrderByValue(query, pair);
        return (query.endAt !== undefined) ?
            value <= query.endAt :
            value < query.endBefore;
    };
}
function getOrderByValue(query, pair) {
    return (query.orderByField !== undefined) ? pair[query.orderByField] : pair.id;
}
function orderByFieldComparer(query) {
    var direction = (query.orderByDirection === "asc") ? 1 : -1;
    var field = query.orderByField;
    return function (a, b) {
        var fa = a.doc.data[field];
        var fb = b.doc.data[field];
        return direction * ((fa < fb) ? -1 : (fa > fb) ? 1 : 0);
    };
}
function orderByIdComparer(query) {
    return function (a, b) {
        var ia = a.id;
        var ib = b.id;
        return ((ia < ib) ? -1 : (ia > ib) ? 1 : 0);
    };
}
function startPredicate(query) {
    return function (pair) {
        var value = getOrderByValue(query, pair);
        return (query.startAt !== undefined) ?
            value >= query.startAt :
            value > query.startAfter;
    };
}
function toPair(doc, id) {
    return { doc: doc, id: id };
}
function wherePredicate(query) {
    var predicates = query.where.map(function (w) { return new Function("pair", "return pair.doc.data[\"" + w.field + "\"] " + w.operator + " " + JSON.stringify(w.value)); });
    return function (pair) { return predicates.every(function (p) { return p(pair); }); };
}
